#include <Arduino.h>
#include <analogPushbuttons.h>
#include <TM1637Display.h>

// pins
volatile byte signal_pin = 4;
volatile byte spindle_pwm_pin = 11;

// speed signal processing and conters
volatile byte pind = 0; // input register D (pins 0-7)
volatile int signalcounter = 0;
volatile int signalcounter_out = 0;
volatile boolean signal, oldsignal;
const byte initialRPM = 4;
byte currentRPM = initialRPM;
byte targetRPM = initialRPM;

volatile int cyclecounter = 0;
const volatile int cycle_threshold_180ms = 180;
volatile int counter_180ms = 0;

volatile boolean trigger_180ms = false;

// 1 count is 333rpm, from there it goes 667, 1000 etc. until 13K
//const short speedVals[] = {0,0,1,1,-1,2,2,-1,-1,3,3,-1,4,4,-1,5,5,-1,-1,6,6,-1,7,7,-1,8,8,-1,-1,9,9,-1,10,10,-1,11,11,-1,-1,12,12,-1,13,13};
const short speedVals[] = {0,0,1,1,1,2,2,2,3,3,3,4,4,4,5,5,5,6,6,6,7,7,7,8,8,8,9,9,9,10,10,10,11,11,11,12,12,12,13,13,13};;

// coarse pwm values for 4k - 11k rpm (used to ramp down)
const byte targetRPMVals[] = {4,5,6,7,8,9,10,11,12};
 // {16,18,21,25,30,35,42,45,60};

const byte pwm4K = 16; // the value where the spindle spins at about 4K
byte currentPWMval = pwm4K;

const byte powerPercentage[] = {0,0,1,1,2,2,2,3,3,4,4,4,5,5,5,6,6,7,7,7,8,8,9,9,9,10,10,11,11,11,12,12,13,13,13,14,14,15,15,15,16,16,16,17,17,18,18,18,19,19,20,20,20,21,21,22,22,22,23,23,24,24,24,25,25,25,26,26,27,27,27,28,28,29,29,29,30,30,31,31,31,32,32,33,33,33,34,34,35,35,35,36,36,36,37,37,38,38,38,39,39,40,40,40,41,41,42,42,42,43,43,44,44,44,45,45,45,46,46,47,47,47,48,48,49,49,49,50,50,51,51,51,52,52,53,53,53,54,54,55,55,55,56,56,56,57,57,58,58,58,59,59,60,60,60,61,61,62,62,62,63,63,64,64,64,65,65,65,66,66,67,67,67,68,68,69,69,69,70,70,71,71,71,72,72,73,73,73,74,74,75,75,75,76,76,76,77,77,78,78,78,79,79,80,80,80,81,81,82,82,82,83,83,84,84,84,85,85,85,86,86,87,87,87,88,88,89,89,89,90,90,91,91,91,92,92,93,93,93,94,94,95,95,95,96,96,96,97,97,98,98,98,99,99,99,99};
byte powerVal = 0;
analogPushbuttons AP(A0, 55);
byte pot_pos_from_char = 0;

TM1637Display display(2, 3);

void setup() {

  // 9 positions for the speed pot
  AP.addButton(10, 'A');
  AP.addButton(67, 'B');
  AP.addButton(185, 'C');
  AP.addButton(313, 'D');
  AP.addButton(458, 'E');
  AP.addButton(615, 'F');
  AP.addButton(760, 'G');
  AP.addButton(885, 'H');
  AP.addButton(1010, 'I');


  DDRD = B00000000;



  TCCR2B = TCCR2B & B11111000 | B00000110; // for PWM frequency for pin 11 of 122.55 Hz
  
  
  
  
  //pinMode(spindle_pwm_pin, OUTPUT);
  Serial.begin(19200);
  display.setBrightness(0x0f);
  
  
  
  
  
  
  cli();//disable interrupts to set timer1 interrupt
  TCCR1A = 0;// set entire TCCR1A register to 0
  TCCR1B = 0;// same for TCCR1B
  TCNT1  = 0;//initialize counter value to 0   // set timer count for 65536khz increments
  OCR1A = 1999;// = (16*10^6) / (1000*8) - 1
  TCCR1B |= (1 << WGM12); // turn on CTC mode
  TCCR1B |= (1 << CS11);     // Set CS11 bit for 8 prescaler
  TIMSK1 |= (1 << OCIE1A); // enable timer compare interrupt
  sei(); // enable interrupts
}
ISR(TIMER1_COMPA_vect){  // the timer-driven interrupt routine
  pind = PIND;
  signal = (pind & (1 << signal_pin)) > 0;

  if (signal && (! oldsignal))
    signalcounter ++;

  oldsignal = signal;

  if (cyclecounter >= cycle_threshold_180ms) {
    trigger_180ms = true;
    cyclecounter = 0;
    signalcounter_out = signalcounter;
    signalcounter = 0;
  }
 
  cyclecounter ++;
}




void loop()
{

//// stuff that needs to run permanently

// asking the current pot position
  boolean temp = AP.keypressed();
  pot_pos_from_char = AP.getCurrentChar();

//// stuff that runs every 180ms
if (trigger_180ms) {

// calculating current spindle speed
currentRPM = speedVals[signalcounter_out];
targetRPM = targetRPMVals[pot_pos_from_char - 65];




// if the spindle does not run, don't try to hit the target value, keep at 4K
if (signalcounter_out < 5) {
    currentPWMval = pwm4K;
    powerVal = 0;
    }






// comparing current speed with target speed    
else {
  # spindle is too slow
  if (currentRPM < targetRPM)
    currentPWMval = currentPWMval + 1;
  # spindle is too fast
  if (currentRPM > targetRPM)
    currentPWMval = currentPWMval - 2;
      






  powerVal = powerPercentage[currentPWMval];
}

if (currentPWMval < pwm4K)
  currentPWMval = pwm4K;

if (currentPWMval > 70)
  currentPWMval = 70;

  Serial.println(currentPWMval);
  analogWrite(spindle_pwm_pin, currentPWMval);

// LCD routine
display.showNumberDec(currentRPM, true, 2, 2);
display.showNumberDec(powerVal, true, 2, 0);

trigger_180ms = false;
}

  



}